"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const internals_1 = require("../internals");
function normalizeIndex(index, size) {
    return index < 0 ? size + index < 0 ? -1 : size + index
        : index >= size ? -1 : index;
}
/**
 * Retrieves the entry at the specified index (ordinal) in the tree. If a negative number is specified for the index,
 * the number is treated as a backtracking offset, with -1 matching the last element in the list, -2 matching the
 * second-last, and so forth.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {number} index The numeric index of the entry to retrieve (A negative number counts as a backtracking offset from the end of the tree)
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(RedBlackTreeEntry<K, V>|undefined)} The tree entry at the specified index, or undefined if the index was out of range
 */
function at(index, tree) {
    return (index = normalizeIndex(index, tree._size)) === -1 ? void 0 : internals_1.findByIndex(index, tree);
}
exports.at = at;
/**
 * Retrieves the key at the specified index (ordinal) in the tree. If a negative number is specified for the index,
 * the number is treated as a backtracking offset, with -1 matching the last element in the list, -2 matching the
 * second-last, and so forth.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {number} index The numeric index of the entry to retrieve (A negative number counts as a backtracking offset from the end of the tree)
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(K|undefined)} The key at the specified index, or undefined if the index was out of range
 */
function keyAt(index, tree) {
    const node = (index = normalizeIndex(index, tree._size)) === -1 ? void 0 : internals_1.findByIndex(index, tree);
    return core_1.isDefined(node) ? node.key : void 0;
}
exports.keyAt = keyAt;
/**
 * Retrieves the value of the entry at the specified index (ordinal) in the tree. If a negative number is specified for
 * the index, the number is treated as a backtracking offset, with -1 matching the last element in the list, -2 matching
 * the second-last, and so forth.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {number} index The numeric index of the entry to retrieve (A negative number counts as a backtracking offset from the end of the tree)
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(V|undefined)} The value at the specified index, or undefined if the index was out of range
 */
function valueAt(index, tree) {
    const node = (index = normalizeIndex(index, tree._size)) === -1 ? void 0 : internals_1.findByIndex(index, tree);
    return core_1.isDefined(node) ? node.value : void 0;
}
exports.valueAt = valueAt;
/**
 * Determines the index (ordinal) of the tree entry that has the specified key. If the key does not exist in the tree, -1 is returned.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {K} key The key of the tree entry to find the index for
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {number} The index of the key in the tree, or -1 if the key was not found
 */
function indexOf(key, tree) {
    return internals_1.findIndex(key, tree._root, tree._compare);
}
exports.indexOf = indexOf;
/**
 * Creates an iterator for which the first entry is at the specified index in the tree. If the index is out of range, an
 * empty iterator is returned.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {boolean} reverse If true, the iterator will iterate backward toward the first entry in the tree
 * @param {number} index The numeric index of the entry to retrieve (A negative number counts as a backtracking offset from the end of the tree)
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {RedBlackTreeIterator<K, V>} An iterator that retrieves each successive entry in the tree, starting from the specified index
 */
function iterateFromIndex(reverse, index, tree) {
    var path = (index = normalizeIndex(index, tree._size)) === -1 ? internals_1.PathNode.NONE : internals_1.findPathToIndex(index, tree._root);
    return internals_1.RedBlackTreeIterator.create(path, tree._compare, reverse);
}
exports.iterateFromIndex = iterateFromIndex;
function iterateValuesFromIndex(reverse, index, tree) {
    return new internals_1.RedBlackTreeValueIterator(iterateFromIndex(reverse, index, tree));
}
exports.iterateValuesFromIndex = iterateValuesFromIndex;
function iterateKeysFromIndex(reverse, index, tree) {
    return new internals_1.RedBlackTreeKeyIterator(iterateFromIndex(reverse, index, tree));
}
exports.iterateKeysFromIndex = iterateKeysFromIndex;

//# sourceMappingURL=at.js.map
